/*
 * Backlight driver for Amstrad E3 (Delta) videophone.
 *
 * Copyright (C) 2006 Jonathan McDowell <noodles@earth.li>
 *
 * Based on Richard Purdie's Corgi driver.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/fb.h>
#include <linux/backlight.h>
#include <linux/platform_device.h>

#include <asm/arch/omap16xx.h>
#include <asm/arch/hardware.h>
#include <asm/arch/board-ams-delta.h>

#define AMS_DELTA_DEFAULT_INTENSITY	90
#define AMS_DELTA_MAX_INTENSITY		255

static int ams_delta_bl_intensity;
static int ams_delta_bl_powermode = FB_BLANK_UNBLANK;

static struct backlight_device *ams_delta_backlight_device;

static void ams_delta_bl_blank(int blank)
{
	switch(blank) {

	case FB_BLANK_NORMAL:
	case FB_BLANK_VSYNC_SUSPEND:
	case FB_BLANK_HSYNC_SUSPEND:
	case FB_BLANK_POWERDOWN:
		if (ams_delta_bl_powermode == FB_BLANK_UNBLANK) {
			ams_delta_latch2_write(AMS_DELTA_LATCH2_LCD_VBLEN, 0);
			ams_delta_bl_powermode = blank;
		}
		break;
	case FB_BLANK_UNBLANK:
		if (ams_delta_bl_powermode != FB_BLANK_UNBLANK) {
			ams_delta_latch2_write(AMS_DELTA_LATCH2_LCD_VBLEN,
				AMS_DELTA_LATCH2_LCD_VBLEN);
			ams_delta_bl_powermode = blank;
		}
		break;
	}
}

static void ams_delta_bl_send_intensity(struct backlight_device *bd)
{
	int intensity = bd->props->brightness;

	if (bd->props->power != FB_BLANK_UNBLANK)
		intensity = 0;
	if (bd->props->fb_blank != FB_BLANK_UNBLANK)
		intensity = 0;

	if (intensity > AMS_DELTA_MAX_INTENSITY)
		intensity=AMS_DELTA_MAX_INTENSITY;

	ams_delta_bl_intensity = intensity;

	omap_writeb(intensity, OMAP_PWL_ENABLE);
}


#ifdef CONFIG_PM
static int ams_delta_bl_suspend(struct platform_device *dev, pm_message_t state)
{
	ams_delta_bl_blank(FB_BLANK_POWERDOWN);
	return 0;
}

static int ams_delta_bl_resume(struct platform_device *dev)
{
	ams_delta_bl_blank(FB_BLANK_UNBLANK);
	return 0;
}
#else
#define ams_delta_bl_suspend	NULL
#define ams_delta_bl_resume	NULL
#endif

static int ams_delta_bl_update_status(struct backlight_device *bd)
{
	ams_delta_bl_send_intensity(ams_delta_backlight_device);
	ams_delta_bl_blank(bd -> props -> fb_blank);

	return 0;
}

static int ams_delta_bl_get_intensity(struct backlight_device *bd)
{
	return ams_delta_bl_intensity;
}

static struct backlight_properties ams_delta_bl_data = {
	.owner		= THIS_MODULE,
	.get_brightness = ams_delta_bl_get_intensity,
	.update_status  = ams_delta_bl_update_status,
};

static int __init ams_delta_bl_probe(struct platform_device *pdev)
{
	ams_delta_backlight_device = backlight_device_register("ams-delta-bl",
			NULL, &ams_delta_bl_data);
	if (IS_ERR(ams_delta_backlight_device))
		return PTR_ERR(ams_delta_backlight_device);

	ams_delta_bl_data.max_brightness = AMS_DELTA_MAX_INTENSITY;

	omap_writeb(1, OMAP_PWL_CLK_ENABLE);
	ams_delta_latch2_write(AMS_DELTA_LATCH2_LCD_VBLEN,
			AMS_DELTA_LATCH2_LCD_VBLEN);

	printk("Amstrad Delta backlight driver initialized.\n");
	return 0;
}

static int ams_delta_bl_remove(struct platform_device *dev)
{
	backlight_device_unregister(ams_delta_backlight_device);

	ams_delta_bl_intensity=0;
	ams_delta_bl_send_intensity(ams_delta_backlight_device);
	omap_writeb(0, OMAP_PWL_CLK_ENABLE);
	ams_delta_latch2_write(AMS_DELTA_LATCH2_LCD_VBLEN, 0);

	printk("Amstrad Delta backlight driver unloaded\n");
	return 0;
}

static struct platform_driver ams_delta_bl_driver = {
	.probe		= ams_delta_bl_probe,
	.remove		= ams_delta_bl_remove,
	.suspend	= ams_delta_bl_suspend,
	.resume		= ams_delta_bl_resume,
	.driver		= {
		.name	= "ams-delta-bl",
	},
};

static int __init ams_delta_bl_init(void)
{
	return platform_driver_register(&ams_delta_bl_driver);
}

static void __exit ams_delta_bl_exit(void)
{
	platform_driver_unregister(&ams_delta_bl_driver);
}

module_init(ams_delta_bl_init);
module_exit(ams_delta_bl_exit);

MODULE_AUTHOR("Jonathan McDowell <noodles@earth.li>");
MODULE_DESCRIPTION("Amstrad Delta backlight driver");
MODULE_LICENSE("GPLv2");
